require 'spec_helper'

module Hikki
  module Adapters
    describe MemoryAdapter do
      subject(:adapter) { MemoryAdapter.new(uuid_generator) }
      let(:uuid_generator) { double :generator, uuid: '12345' }
      let(:collection) { 'collection1' }

      describe '#index' do
        it 'returns true' do
          expect(adapter.index(collection, :field1)).to be_true
        end

        it 'creates the index in the store' do
          adapter.index(collection, :field1)
          expect(adapter.store['collection1_index']['field1']).to eq Hash.new
        end
      end

      describe '#save' do
        context 'when an id is provided in the data' do
          let(:data) { { id: id, field1: 'test', field2: 123 } }
          let(:expected) { { 'id' => id, 'field1' => 'test', 'field2' => 123 } }
          let(:id) { '1' }

          it 'returns the data' do
            expect(adapter.save(collection, data)).to eq expected
          end

          it 'persists the data in the store' do
            adapter.save(collection, data)
            expect(adapter.store['collection1']['1']).to eq expected
          end
        end

        context 'when an id is not provided in the data' do
          let(:data) { { field1: 'test', field2: 123 } }
          let(:expected) { { 'id' => id, 'field1' => 'test', 'field2' => 123 } }
          let(:id) { '12345' }

          it 'returns the data with the id added' do
            expect(adapter.save(collection, data)).to eq expected
          end

          it 'persists the data in the store' do
            adapter.save(collection, data)
            expect(adapter.store['collection1']['12345']).to eq expected
          end
        end

        context 'when an index exists on a field in the data' do
          let(:data) { { id: id, field1: 'test', field2: 123 } }
          let(:expected) { { 'id' => id, 'field1' => 'test', 'field2' => 123 } }
          let(:id) { '1' }
          before { adapter.index(collection, :field1) }

          it 'adds an entry in the index' do
            adapter.save(collection, data)
            expect(adapter.store['collection1_index']['field1']['test']).to eq ['1']
          end
        end
      end

      describe '#find' do
          let(:id) { '1' }

        context 'when the id exists' do
          let(:data) { { id: id, field1: 'test', field2: 123 } }
          let(:expected) { { 'id' => id, 'field1' => 'test', 'field2' => 123 } }
          before { adapter.save(collection, data) }

          it 'retrieves the data' do
            expect(adapter.find(collection, id)).to eq expected
          end
        end

        context 'when the id does not exist' do
          let(:expected) { {} }

          it 'returns an empty hash' do
            expect(adapter.find(collection, id)).to eq expected
          end
        end
      end

      describe '#all' do
        context 'with a record in the collection' do
          let(:data) { { id: '1', field1: 'test', field2: 123 } }
          let(:expected) { { 'id' => '1', 'field1' => 'test', 'field2' => 123 } }
          before { adapter.save(collection, data) }

          it 'returns an array containing the record' do
            expect(adapter.all(collection)).to eq [expected]
          end
        end

        context 'with no records in the collection' do
          it 'returns an array containing the record' do
            expect(adapter.all(collection)).to eq []
          end
        end

        context 'with multiple records and paging options specified' do
          before do
            10.times do |i|
              adapter.save(collection, {id: i+1, a: 1})
            end
          end

          context 'with limit 2' do
            let(:limit) { 2 }

            it 'returns only 2 records' do
              expect(adapter.all(collection, {limit: 2}).count).to eq 2
            end

            context 'with an offset of 4' do
              it 'returns the 2 records starting with the offset' do
                expect(adapter.all(collection, {limit: 2, offset: 4})).to eq [{'id' => '5', 'a' => 1}, {'id' => '6', 'a' => 1}]
              end
            end
          end
        end
      end

      describe '#find_by' do
        context 'with a record in the collection matching' do
          let(:data) { { id: '1', field1: 'test', field2: 123 } }
          let(:expected) { { 'id' => '1', 'field1' => 'test', 'field2' => 123 } }

          context 'without an index' do
            before { adapter.save(collection, data) }

            it 'returns an array containing the record' do
              expect(adapter.find_by(collection, :field2, 123)).to eq [expected]
            end
          end

          context 'with an index' do
            before do
              adapter.index(collection, :field1)
              adapter.save(collection, data)
            end

            it 'returns an array containing the record' do
              expect(adapter.find_by(collection, :field1, 'test')).to eq [expected]
            end
          end
        end

        context 'with no matching records in the collection' do
          it 'returns an array containing the record' do
            expect(adapter.find_by(collection, :field1, 'foo')).to eq []
          end
        end

        context 'with multiple matching records and paging options specified' do
          before do
            10.times do |i|
              adapter.save(collection, {id: i+1, a: 1})
            end
          end

          context 'with limit 2' do
            let(:limit) { 2 }

            it 'returns only 2 records' do
              expect(adapter.find_by(collection, :a, 1, {limit: 2}).count).to eq 2
            end

            context 'with an offset of 4' do
              it 'returns the 2 records starting with the offset' do
                expect(adapter.find_by(collection, :a, 1, {limit: 2, offset: 4})).to eq [{'id' => '5', 'a' => 1}, {'id' => '6', 'a' => 1}]
              end
            end
          end
        end
      end

      describe '#remove' do
        context 'with a record for the id in the collection' do
          let(:data) { { id: id, field1: 'test', field2: 123 } }
          let(:id) { '1' }
          before { adapter.save(collection, data) }

          it 'returns true' do
            expect(adapter.remove(collection, id)).to be_true
          end

          it 'removes the record from the store' do
            adapter.remove(collection, id)
            expect(adapter.store[collection][id]).to be_nil
          end

          context 'when there is an index' do
            before do
              adapter.index(collection, :field1)
              adapter.save(collection, data)
            end

            it 'removes the id from the index' do
              expect(adapter.store[collection + '_index']['field1']['test']).to eq ['1']
              adapter.remove(collection, id)
              expect(adapter.store[collection + '_index']['field1']['test']).to eq []
            end
          end
        end
      end

      describe '#remove_all' do
        context 'with a record in the collection' do
          let(:data) { { id: id, field1: 'test', field2: 123 } }
          let(:id) { '1' }
          before { adapter.save(collection, data) }

          it 'returns true' do
            expect(adapter.remove_all(collection)).to be_true
          end

          it 'removes all records from the store' do
            adapter.remove_all(collection)
            expect(adapter.store[collection]).to be_nil
          end

          context 'when there is an index' do
            before do
              adapter.index(collection, :field1)
              adapter.save(collection, data)
            end

            it 'removes the id from the index' do
              expect(adapter.store[collection + '_index']['field1']['test']).to eq ['1']
              adapter.remove_all(collection)
              expect(adapter.store[collection + '_index']).to be_nil
            end
          end
        end
      end
    end
  end
end