require 'spec_helper'

module Hikki
  describe Repository do
    subject(:repository) { Repository.new(writers, readers) }
    let(:writers) { [writer] }
    let(:readers) { [reader] }
    let(:writer) { double :writer, save: mod_data, index: true, remove: true, remove_all: true }
    let(:another_writer) { double :writer, save: mod_data, index: true, remove: true, remove_all: true }
    let(:reader) { double :reader }
    let(:collection) { 'collection' }
    let(:data) { { name: 'test' } }
    let(:mod_data) { { 'id' => '1', 'name' => 'test' } }

    describe '.new' do
      context 'with no readers specified' do
        subject(:repository) { Repository.new(writers) }

        it 'defaults to the first writer for reading' do
          expect(repository.readers.first).to eq repository.writers.first
        end
      end

      context 'with no writers specified' do
        subject(:repository) { Repository.new }

        it 'defaults to the memory adapter for writing' do
          expect(repository.writers.first.class).to eq Hikki::Adapters::MemoryAdapter
        end

        it 'defaults to the first writer for reading' do
          expect(repository.readers.first).to eq repository.writers.first
        end
      end
    end

    describe '#save' do
      it 'calls save on the writer' do
        repository.save(collection, data)
        expect(writer).to have_received(:save).with(collection, data)
      end

      it 'returns the data' do
        expect(repository.save(collection, data)).to eq mod_data
      end

      context 'with a second writer' do
        before { repository.writers << another_writer }

        it 'calls save on both of the writers' do
          expect(writer).to receive(:save).with(collection, data).and_return(mod_data)
          repository.save(collection, data)
          expect(another_writer).to have_received(:save).with(collection, mod_data)
        end

        it 'returns the data' do
          expect(repository.save(collection, data)).to eq mod_data
        end
      end
    end

    describe '#index' do
      it 'calls index on the writer' do
        repository.index(collection, :field)
        expect(writer).to have_received(:index).with(collection, :field)
      end

      context 'with a second writer' do
        before { repository.writers << another_writer }

        it 'calls index on both of the writers' do
          repository.index(collection, :field)
          expect(writer).to have_received(:index).with(collection, :field)
          expect(another_writer).to have_received(:index).with(collection, :field)
        end
      end
    end

    describe '#remove' do
      it 'calls index on the writer' do
        repository.remove(collection, '1')
        expect(writer).to have_received(:remove).with(collection, '1')
      end

      context 'with a second writer' do
        before { repository.writers << another_writer }

        it 'calls index on both of the writers' do
          repository.remove(collection, '1')
          expect(writer).to have_received(:remove).with(collection, '1')
          expect(another_writer).to have_received(:remove).with(collection, '1')
        end
      end
    end

    describe '#remove_all' do
      it 'calls remove_all on the writer' do
        repository.remove_all(collection)
        expect(writer).to have_received(:remove_all).with(collection)
      end

      context 'with a second writer' do
        before { repository.writers << another_writer }

        it 'calls remove_all on both of the writers' do
          repository.remove_all(collection)
          expect(writer).to have_received(:remove_all).with(collection)
          expect(another_writer).to have_received(:remove_all).with(collection)
        end
      end
    end

    describe '#find' do
      context 'when the adapter returns a result' do
        before { reader.stub(:find).with(collection, '1').and_return(data) }

        it 'returns the result' do
          expect(repository.find(collection, '1')).to eq data
        end
      end

      context 'when the adapter does not return a result' do
        before { reader.stub(:find).with(collection, '1').and_return({}) }

        it 'returns {}' do
          expect(repository.find(collection, '1')).to eq Hash.new
        end
      end

      context 'with 2 readers' do
        let(:another_reader) { double :reader }
        before { repository.readers << another_reader }

        context 'and the record is found in the first' do
          before do
            reader.stub(:find).with(collection, '1').and_return(data)
            another_reader.stub(:find).with(collection, '1').and_return({})
          end

          it 'returns the result from the first reader' do
            expect(repository.find(collection, '1')).to eq data
          end
        end

        context 'and the record is found only in the second' do
          before do
            reader.stub(:find).with(collection, '1').and_return({})
            another_reader.stub(:find).with(collection, '1').and_return(data)
          end

          it 'returns the result from the second reader' do
            expect(repository.find(collection, '1')).to eq data
          end
        end
      end
    end

    describe '#all' do
      context 'when the adapter returns the results' do
        before { reader.stub(:all).with(collection, {}).and_return([data]) }

        it 'returns the results' do
          expect(repository.all(collection)).to eq [data]
        end
      end

      context 'when the adapter does not return a result' do
        before { reader.stub(:all).with(collection, {}).and_return([]) }

        it 'returns {}' do
          expect(repository.all(collection)).to eq []
        end
      end

      context 'with 2 readers' do
        let(:another_reader) { double :reader }
        before { repository.readers << another_reader }

        context 'and the record is found in the first' do
          before do
            reader.stub(:all).with(collection, {}).and_return([data])
            another_reader.stub(:all).with(collection, {}).and_return([])
          end

          it 'returns the result from the first reader' do
            expect(repository.all(collection)).to eq [data]
          end
        end

        context 'and the record is found only in the second' do
          before do
            reader.stub(:all).with(collection, {}).and_return([])
            another_reader.stub(:all).with(collection, {}).and_return([data])
          end

          it 'returns the result from the second reader' do
            expect(repository.all(collection)).to eq [data]
          end
        end
      end

      context 'with options' do
        before { reader.stub(:all).with(collection, {limit: 10}).and_return([data]) }

        it 'returns the results' do
          expect(repository.all(collection, limit: 10)).to eq [data]
        end
      end
    end

    describe '#find_by' do
      context 'when the adapter returns the results' do
        before { reader.stub(:find_by).with(collection, :field, 'abc', {}).and_return([data]) }

        it 'returns the results' do
          expect(repository.find_by(collection, :field, 'abc')).to eq [data]
        end
      end

      context 'when the adapter does not return a result' do
        before { reader.stub(:find_by).with(collection, :field, 'abc', {}).and_return([]) }

        it 'returns {}' do
          expect(repository.find_by(collection, :field, 'abc')).to eq []
        end
      end

      context 'with 2 readers' do
        let(:another_reader) { double :reader }
        before { repository.readers << another_reader }

        context 'and the record is found in the first' do
          before do
            reader.stub(:find_by).with(collection, :field, 'abc', {}).and_return([data])
            another_reader.stub(:find_by).with(collection, :field, 'abc', {}).and_return([])
          end

          it 'returns the result from the first reader' do
            expect(repository.find_by(collection, :field, 'abc')).to eq [data]
          end
        end

        context 'and the record is found only in the second' do
          before do
            reader.stub(:find_by).with(collection, :field, 'abc', {}).and_return([])
            another_reader.stub(:find_by).with(collection, :field, 'abc', {}).and_return([data])
          end

          it 'returns the result from the second reader' do
            expect(repository.find_by(collection, :field, 'abc')).to eq [data]
          end
        end
      end

      context 'with options' do
        before { reader.stub(:find_by).with(collection, :field, 'abc', {limit: 10}).and_return([data]) }

        it 'returns the results' do
          expect(repository.find_by(collection, :field, 'abc', limit: 10)).to eq [data]
        end
      end
    end

  end
end